@@ -3,19 +3,21 @@
 
 double jeffmlikeNWayAov(double *XtCX, double *XtCy, double ytCy, int N, int P, double *g)
 {
-	double *W,ldetS=0,ldetW,top,bottom1,bottom2,q;
+	double *W,*Winv,ldetS=0,ldetW,top,bottom1,bottom2,q;
 	int i = 0, Psqr=P*P;
 	
 	W = Memcpy(Calloc(Psqr,double),XtCX,Psqr);
 	
-	for(i=0;i<P;i++)
+  for(i=0;i<P;i++)
 	{
 		ldetS += log(g[i]);
 		W[i + P*i] = W[i + P*i] + 1/g[i];
 	}
-	InvMatrixUpper(W, P);
+  Winv = Memcpy(Calloc(Psqr,double),W,Psqr);
+
+  InvMatrixUpper(W, P);
 	internal_symmetrize(W, P);
-	ldetW = matrixDet(W, P, P, 1);
+	ldetW = -matrixDet(Winv, P, P, 1);
 	
 	q = quadform(XtCy, W, P, 1, P);
 	
@@ -26,6 +28,7 @@ double jeffmlikeNWayAov(double *XtCX, double *XtCy, double ytCy, int N, int P, d
 	bottom2 = 0.5*ldetS;
 	
 	Free(W);
+  Free(Winv);
 	
 	return(top-bottom1-bottom2);	
 }
@@ -101,11 +104,9 @@ double jeffSamplerNwayAov(double *samples, double *gsamples, int iters, double *
 	
 }
 
-
-
 SEXP RjeffSamplerNwayAov(SEXP Riters, SEXP RXtCX, SEXP RXtCy, SEXP RytCy, SEXP RN, SEXP RP, SEXP RnGs, SEXP RgMap, SEXP Ra, SEXP Rb, SEXP progressR, SEXP pBar, SEXP rho)
 {
-	double *a,*b,*XtCy,*XtCX,ytCy,*samples,*avg;
+  double *a,*b,*XtCy,*XtCX,ytCy,*samples,*avg;
 	int iters,nGs,*gMap,N,P,progress;
 
 	SEXP Rsamples,Rgsamples,Ravg,returnList;
@@ -143,6 +144,108 @@ SEXP RjeffSamplerNwayAov(SEXP Riters, SEXP RXtCX, SEXP RXtCy, SEXP RytCy, SEXP R
 	return(returnList);
 }
 
+
+double importanceSamplerNwayAov(double *samples, double *qsamples, int iters, double *XtCX, double *XtCy, double ytCy, int N, int P,int nGs, int *gMap, double *a, double *b, double *mu, double *sig, int progress, SEXP pBar, SEXP rho)
+{
+  int i=0,j=0;
+	double avg = 0, *q1, g2[P], sumq=0, sumdinvgamma=0, sumdnorm=0;
+	
+	// progress stuff
+	SEXP sampCounter, R_fcall;
+	int *pSampCounter;
+    PROTECT(R_fcall = lang2(pBar, R_NilValue));
+	PROTECT(sampCounter = NEW_INTEGER(1));
+	pSampCounter = INTEGER_POINTER(sampCounter);
+
+	
+	for(i=0;i<iters;i++)
+	{
+		R_CheckUserInterrupt();
+		
+		//Check progress
+		if(progress && !((i+1)%progress)){
+			pSampCounter[0]=i+1;
+			SETCADR(R_fcall, sampCounter);
+			eval(R_fcall, rho); //Update the progress bar
+		}
+
+		q1 = qsamples + i*nGs;
+    
+	  sumq = 0;
+    sumdinvgamma = 0;
+    sumdnorm = 0;
+		for(j=0;j<nGs;j++)
+		{
+			q1[j]  = rnorm(mu[j],sig[j]);
+		  sumq += q1[j];
+      sumdinvgamma += dgamma(exp(-q1[j]), a[j], 1/b[j], 1) - 2*q1[j];
+      sumdnorm += dnorm(q1[j], mu[j], sig[j], 1);
+    }
+    
+		for(j=0;j<P;j++)
+		{
+			g2[j] = exp(q1[gMap[j]]);
+		}
+	
+		samples[i] = jeffmlikeNWayAov(XtCX, XtCy, ytCy, N, P, g2) + sumq + sumdinvgamma - sumdnorm;
+		if(i==0)
+		{
+			avg = samples[i];
+		}else{
+			avg = LogOnePlusExpX(samples[i]-avg)+avg;
+			//avg = LogOnePlusX(exp(avg-samples[i]))+samples[i];
+		}
+	}
+	
+	UNPROTECT(2);
+	
+	return(avg-log(iters));
+	
+}
+
+SEXP RimportanceSamplerNwayAov(SEXP Riters, SEXP RXtCX, SEXP RXtCy, SEXP RytCy, SEXP RN, SEXP RP, SEXP RnGs, SEXP RgMap, SEXP Ra, SEXP Rb, SEXP Rmu, SEXP Rsig, SEXP progressR, SEXP pBar, SEXP rho)
+{
+	double *a,*b,*XtCy,*XtCX,ytCy,*samples,*avg, *mu, *sig;
+	int iters,nGs,*gMap,N,P,progress;
+
+	SEXP Rsamples,Rgsamples,Ravg,returnList;
+	
+	iters = INTEGER_VALUE(Riters);
+	XtCX = REAL(RXtCX);
+	XtCy = REAL(RXtCy);
+	ytCy = REAL(RytCy)[0];
+	nGs = INTEGER_VALUE(RnGs);
+	gMap = INTEGER_POINTER(RgMap);
+	a = REAL(Ra);
+	b = REAL(Rb);
+	N = INTEGER_VALUE(RN);
+	P = INTEGER_VALUE(RP);
+  mu = REAL(Rmu);
+	sig = REAL(Rsig);
+
+  progress = INTEGER_VALUE(progressR);
+	
+	PROTECT(Rsamples = allocVector(REALSXP,iters));
+	PROTECT(Ravg = allocVector(REALSXP,1));
+	PROTECT(returnList = allocVector(VECSXP,3));
+	PROTECT(Rgsamples = allocMatrix(REALSXP,nGs,iters));	
+
+	samples = REAL(Rsamples);
+	avg = REAL(Ravg);
+	
+	GetRNGstate();
+	avg[0] = importanceSamplerNwayAov(samples, REAL(Rgsamples), iters, XtCX, XtCy, ytCy, N, P, nGs, gMap, a, b, mu, sig, progress, pBar, rho);
+	PutRNGstate();
+	
+	SET_VECTOR_ELT(returnList, 0, Ravg);
+    SET_VECTOR_ELT(returnList, 1, Rsamples);
+    SET_VECTOR_ELT(returnList, 2, Rgsamples);
+	
+	UNPROTECT(4);
+	
+	return(returnList);
+}
+
 SEXP RGibbsNwayAov(SEXP Riters, SEXP Ry, SEXP RX, SEXP RXtX, SEXP RXty, SEXP RN, SEXP RP, SEXP RnGs, SEXP RgMap, SEXP Rr, SEXP progressR, SEXP pBar, SEXP rho)
 {
 	double *a,*b,*Xty,*XtX,*samples,*X, *y, *r;
